// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/gpio_read.c $
// $Rev: 31452 $
// $Date: 2014-11-13 09:58:59 -0600 (Thu, 13 Nov 2014) $

/*****************************************************************************
*
*	Implement support for GPIO Read service.
*
*****************************************************************************/

#include <windows.h>
#include <stdio.h>

#include "gui.h"
#include "gpio.h"
#include "align.h"
#include "button.h"
#include "resource.h"
#include "util_ctrl.h"



/* #defines	*****************************************************************/

#define	SECTION_NAME		"gpio read"



/*****************************************************************************
*
*	Function:	_refresh_window
*
*	Purpose:
*
*		Update the controls based on the current state.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		device		The device to access.
*
*		msg			A status message to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _refresh_window(HWND dlg, gpio_xfer_t* data)
{
	BOOL	access;
	HWND	control;
	BOOL	inactive	= data->active == FALSE;
	BOOL	idle_data	= (data->values) && inactive;

	if (data->device->handle)
		access	= TRUE;
	else
		access	= FALSE;

	control	= GetDlgItem(dlg, IDC_GPIO_XFER_START);
	EnableWindow(control, (BOOL) ((access) && (inactive)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_FILE);
	EnableWindow(control, idle_data);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_TEXT);
	EnableWindow(control, (BOOL) ((access) && (inactive)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_INTERVAL_LIST);
	EnableWindow(control, (BOOL) ((access) && (inactive)));
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_SCALE_TEXT);
	EnableWindow(control, idle_data);
	control	= GetDlgItem(dlg, IDC_GPIO_XFER_SCALE_SCROLL);
	EnableWindow(control, idle_data);
}



/*****************************************************************************
*
*	Function:	_wm_initdialog
*
*	Purpose:
*
*		Process a WM_INITDIALOG message for the dialog.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _wm_initdialog(HWND dlg, gpio_xfer_t* data)
{
	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_START, "Read");
	ctrl_menu_set_text(dlg, IDC_GPIO_XFER_FILE, "Save");
}



/*****************************************************************************
*
*	Function:	_xfer_begin
*
*	Purpose:
*
*		Prepare for data transfer. This is called before the transfer thread
*		is started.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		TRUE	Continue with data transfer.
*		FALSE	Don't continue with data transfer.
*
*****************************************************************************/

static int _xfer_begin(gpio_xfer_t* data)
{
	data->transfer	= data->capacity;
	data->values	= 0;
	ctrl_menu_set(data->dlg, IDC_GPIO_XFER_STOP, CTRL_ENABLE | CTRL_FOCUS);
	return(TRUE);
}



/*****************************************************************************
*
*	Function:	_xfer_data
*
*	Purpose:
*
*		Perform a single data value transfer request.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		GSC_SUCCESS	All went well. Continue with data transfer.
*		else		An appropriate error status. Terminate transfer.
*
*****************************************************************************/

static U32 _xfer_data(gpio_xfer_t* data)
{
	U32	status;

	status	= hpdi32_gpio_read(data->device->handle, &data->buffer[data->values]);
	return(status);
}



/*****************************************************************************
*
*	Function:	_xfer_end
*
*	Purpose:
*
*		Cleanup after data transfer. This is called from the transfer thread
*		if the thread is started. Otherwise it is called from the main thread.
*		This is called only if xfer_begin() indicates that the transfer should
*		procede.
*
*	Arguments:
*
*		data	The dialog specific data we track.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

static void _xfer_end(gpio_xfer_t* data)
{
	ctrl_menu_set(data->dlg, IDC_GPIO_XFER_START, CTRL_ENABLE | CTRL_FOCUS);
}



/*****************************************************************************
*
*	Function:	_xfer_file
*
*	Purpose:
*
*		Prompt for a file name and save the data.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		data		The window specific data we track.
*
*	Returned:
*
*		TRUE		Apply the INTERVAL value that is currently selected?
*		FALSE		Use the existing interval value.
*
*****************************************************************************/

static int _xfer_file(HWND dlg, gpio_xfer_t* data)
{
	TCHAR			buf[128 + _MAX_PATH];
	char			dir[_MAX_PATH];
	FILE*			file;
	TCHAR			name[_MAX_PATH];
	int				i;
	OPENFILENAME	ofn;

	strcpy(name, "gpio.bin");
	GetCurrentDirectory(sizeof(dir), dir);

	// Initialize OPENFILENAME
	memset(&ofn, 0, sizeof(OPENFILENAME));
	ofn.lStructSize		= sizeof(OPENFILENAME);
	ofn.hwndOwner		= dlg;
	ofn.lpstrFilter		= "Binary Files (*.bin)\0*.bin\0"
							"All (*.*)\0*.*\0";
	ofn.nFilterIndex	= 1;
	ofn.lpstrFile		= name;
	ofn.nMaxFile		= sizeof(name);
	ofn.lpstrInitialDir	= dir;
	ofn.lpstrTitle		= "Save GPIO Data";
	ofn.Flags			= OFN_PATHMUSTEXIST
						| OFN_OVERWRITEPROMPT
						| OFN_HIDEREADONLY;
	i					= GetSaveFileName(&ofn);

	for (; i;)
	{
		file	= fopen(name, "wb");

		if (file == NULL)
		{
			sprintf(buf, "Unable to create '%s'.", name);
			ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, buf);
			break;
		}

		i	= (int) fwrite(data->buffer, 1, (SIZE_T) data->values, file);

		if (i != data->values)
			sprintf(buf, "Write failure to '%s'.", name);
		else
			sprintf(buf, "Saved %d bytes to '%s'.", data->values, name);

		fclose(file);
		ctrl_menu_set_text(dlg, IDC_GPIO_XFER_STATUS, buf);
		break;
	}

	return(FALSE);
}



/*****************************************************************************
*
*	Function:	_service_init
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_INIT service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_init(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static const align_t list[]	=
	{
		{
		/* id		*/	IDC_GPIO_READ,
		/* ref		*/	IDC_RX_ENABLE,
		/* flags	*/	ALIGN_COPY,
		/* value	*/	0
		},
		{
		/* id		*/	IDC_GPIO_READ,
		/* ref		*/	IDC_GPIO_0,
		/* flags	*/	ALIGN_T_T | ALIGN_H,
		/* value	*/	0
		},
		{ 0, 0, 0, 0 }
	};

	align_control_list(dlg, list);
	button_init(dlg, device, IDC_GPIO_READ);
	return(0);
}



/*****************************************************************************
*
*	Function:	_service_refresh
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_REFRESH service for
*		the GUI window.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_refresh(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	if (device->handle)
		ctrl_menu_set(dlg, IDC_GPIO_READ, CTRL_ENABLE);
	else
		ctrl_menu_set(dlg, IDC_GPIO_READ, CTRL_DISABLE);

	return(0);
}



/*****************************************************************************
*
*	Function:	_service_wm_command
*
*	Purpose:
*
*		Perform the actions appropriate for the SERVICE_WM_COMMAND service.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0
*
*****************************************************************************/

static LRESULT _service_wm_command(
	HWND		dlg,
	device_t*	device,
	WPARAM		wParam,
	LPARAM		lParam)
{
	static	U8			buffer[3000];
	static	gpio_xfer_t	data;
	LRESULT				ret;

	data.buffer			= buffer;
	data.capacity		= (int) sizeof(buffer);
	data.idc_interval	= FALSE;
	data.refresh_window	= _refresh_window;
	data.section		= SECTION_NAME;
	data.title			= "GPIO Read";
	data.wm_initdialog	= _wm_initdialog;
	data.xfer_begin		= _xfer_begin;
	data.xfer_data		= _xfer_data;
	data.xfer_end		= _xfer_end;
	data.xfer_file		= _xfer_file;

	ret	= gpio_xfer_service(dlg, device, SERVICE_WM_COMMAND, (ULONG) &data, lParam);
	return(ret);
}



/*****************************************************************************
*
*	Function:	gpio_read_service
*
*	Purpose:
*
*		Implement the common service entry point for the GPIO Read services.
*
*	Arguments:
*
*		dlg		The dialog handle.
*
*		device	The device to access.
*
*		service	The service to perform.
*
*		wParam	A service dependent value.
*
*		lParam	A service dependent value.
*
*	Returned:
*
*		0		No service was called.
*		else	The value from the called service, which may be zero.
*
*****************************************************************************/

LRESULT gpio_read_service(
	HWND			dlg,
	device_t*		device,
	service_type_t	service,
	WPARAM			wParam,
	LPARAM			lParam)
{
	static const service_call_t	list[]	=
	{
		/* service				function			*/
		{ SERVICE_INIT,			_service_init		},
		{ SERVICE_REFRESH,		_service_refresh	},
		{ SERVICE_WM_COMMAND,	_service_wm_command	},
		{ SERVICE_END_LIST,		NULL				}
	};

	LRESULT	ret;

	ret	= service_call(dlg, device, list, service, wParam, lParam);
	return(ret);
}


