// HPDI32 - Application - Win32 - Any Compiler
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/SDK/hpdi32/visualhpdi32/win32/util_ctrl.c $
// $Rev: 2902 $
// $Date: 2009-08-09 17:48:49 -0500 (Sun, 09 Aug 2009) $

/*****************************************************************************
*
*	Implement various utility service for window controls.
*
*****************************************************************************/

#include <windows.h>

#include "util_ctrl.h"



/*****************************************************************************
*
*	Function:	ctrl_hex_2_u32
*
*	Purpose:
*
*		Convert a control's text to a U32. The text must be in hex form, and
*		it may include a prefix of "0x".
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		ctrl_id		The id of the control to access.
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 ctrl_hex_2_u32(HWND dlg, int ctrl_id, U32* u32)
{
	char	buf[1024];
	U32		status;

	GetDlgItemText(dlg, ctrl_id, buf, sizeof(buf));
	status	= hex_text_2_u32(buf, u32);
	return(status);
}



/*****************************************************************************
*
*	Function:	ctrl_text_2_u32
*
*	Purpose:
*
*		Convert a control's text to a U32. The number may be in hex form or
*		decimal form, which may include a K, M or G suffix.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		ctrl_id		The id of the control to access.
*
*		k			The size of a single 1K unit.
*
*		u32			The value is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 ctrl_text_2_u32(HWND dlg, int ctrl_id, int k, U32* u32)
{
	char	buf[1024];
	U32		status;

	GetDlgItemText(dlg, ctrl_id, buf, sizeof(buf));
	status	= numeric_text_2_u32(buf, k, u32);
	return(status);
}



/*****************************************************************************
*
*	Function:	ctrl_radio_2_u32
*
*	Purpose:
*
*		Convert a radio button list selection to a U32.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		list		The list of radio buttons to access.
*
*		size		The size of the list.
*
*		defalt		The value to apply if no selection is found.
*
*	Returned:
*
*		The value corresponding to the selection.
*
*****************************************************************************/

U32 ctrl_radio_2_u32(HWND dlg, const ctrl_id_u32_t* list, size_t size, U32 defalt)
{
	HWND	control;
	int		i;
	LRESULT	state;
	U32		u32		= defalt;

	for (i = 0; i < (int) size; i++)
	{
		control	= GetDlgItem(dlg, list[i].ctrl_id);

		if (control == NULL)
			continue;

		state	= SendMessage(control, BM_GETCHECK, 0, 0);

		if (state == BST_CHECKED)
		{
			u32	= list[i].u32;
			break;
		}
	}

	return(u32);
}



/*****************************************************************************
*
*	Function:	ctrl_u32_2_radio
*
*	Purpose:
*
*		Convert a U32 value to a radio button selection option.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		u32			The value to apply.
*
*		list		The list of radio buttons to access.
*
*		size		The size of the list.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void ctrl_u32_2_radio(HWND dlg, U32 u32, const ctrl_id_u32_t* list, size_t size)
{
	HWND	control;
	int		i;
	int		selected	= FALSE;

	for (i = 0; i < (int) size; i++)
	{
		control	= GetDlgItem(dlg, list[i].ctrl_id);

		if (control == NULL)
			continue;

		if ((u32 != list[i].u32) || (selected) || (u32 == (U32) -1))
		{
			SendMessage(control, BM_SETCHECK, BST_UNCHECKED, 0);
		}
		else
		{
			SendMessage(control, BM_SETCHECK, BST_CHECKED, 0);
			selected	= TRUE;
		}
	}
}



/*****************************************************************************
*
*	Function:	ctrl_menu_set_text
*
*	Purpose:
*
*		Update the text of the given control and/or menu item. At present,
*		the '&' characters are stripped from control text.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		id			The id of the item to access.
*
*		text		The text to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void ctrl_menu_set_text(HWND dlg, int id, const char* text)
{
	char	buf[1024];
	HWND	control	= GetDlgItem(dlg, id);
	int		i;
	HMENU	menu	= GetMenu(dlg);
	char*	psz		= buf;

	ModifyMenu(menu, (UINT) id, MF_BYCOMMAND | MF_STRING, (UINT) id, text);

	if (text)
	{
		for (i = 0;; i++)
		{
			if (text[i] == 0)
			{
				psz[0]	= 0;
				break;
			}

			if (text[i] == '&')
				continue;

			psz[0]	= text[i];
			psz++;
		}

		SetWindowText(control, buf);
	}
}



/*****************************************************************************
*
*	Function:	ctrl_menu_set
*
*	Purpose:
*
*		Update the state of a menu item and/or control.
*
*	Arguments:
*
*		dlg			The dialog handle.
*
*		id			The id of the item to access.
*
*		flags		The set of flags to apply.
*
*	Returned:
*
*		None.
*
*****************************************************************************/

void ctrl_menu_set(HWND dlg, int id, unsigned int flags)
{
	HWND	control	= GetDlgItem(dlg, id);
	HMENU	menu	= GetMenu(dlg);

	if (flags & CTRL_ENABLE)
	{
		EnableMenuItem(menu, (UINT) id, MF_BYCOMMAND | MF_ENABLED);
		EnableWindow(control, TRUE);
	}
	else if (flags & CTRL_DISABLE)
	{
		EnableMenuItem(menu, (UINT) id, MF_BYCOMMAND | MF_GRAYED);
		EnableWindow(control, FALSE);
	}

	if (flags & CTRL_BM_CHECK)
	{
		CheckMenuItem(menu, (UINT) id, MF_BYCOMMAND | MF_CHECKED);
		DrawMenuBar(dlg);
		SendMessage(control, BM_SETCHECK, BST_CHECKED, 0);
	}
	else if (flags & CTRL_BM_UNCHECK)
	{
		CheckMenuItem(menu, (UINT) id, MF_BYCOMMAND | MF_UNCHECKED);
		DrawMenuBar(dlg);
		SendMessage(control, BM_SETCHECK, BST_UNCHECKED, 0);
	}

	if (flags & CTRL_NULL)
	{
		ModifyMenu(menu, (UINT) id, MF_BYCOMMAND | MF_STRING, (UINT) id, NULL);
		SetWindowText(control, NULL);
	}

	if (flags & CTRL_LB_RESET)
		SendMessage(control, LB_RESETCONTENT, 0, 0);

	if (flags & CTRL_FOCUS)
	{
		EnableWindow(control, TRUE);
		SetFocus(control);
	}

	if (flags & CTRL_DELETE)
	{
		DeleteMenu(menu, (UINT) id, MF_BYCOMMAND);
		DestroyWindow(control);
	}
}


